@@ -1,95 +1,53 @@
 ################################################################################
+# dscale puede ser una funcion o un valor
+################################################################################
 depth.modep=function(lfdata,lfdataref=lfdata,h=NULL,metric,par.metric=list(),
-method="euclidean",scale=FALSE,trim=0.25,draw=FALSE,ask=FALSE) {
+method="euclidean",scale=FALSE,trim=0.15,draw=FALSE,ask=FALSE) {
+equal<-identical(lfdata,lfdataref)
 if (class(lfdata)=="list"){
- lenl<-length(lfdata)
- lenl2<-length(lfdataref)
-  m0<-nrow(lfdata[[1]])
-  if (is.null(rownames(lfdata[[1]]$data)))  rownames(lfdata[[1]]$data)<-1:m0
- nms<-rownames(lfdata[[1]]$data)
- nas<-NULL
-for (i in 1:lenl) {
-  nas<-c(nas,na.action(na.omit(lfdata[[i]])))
-} 
-
-nas<-unique(nas)
-nullans<-!is.null(nas) 
-for (i in 1:lenl) {
-  if (nullans) lfdata[[i]]<-lfdata[[i]][-nas]
-#  lfdataref[[i]]<-lfdataref[[i]][-nas]  
-}
- n<-nrow(lfdata[[1]])
- m<-nrow(lfdataref[[1]]) 
- mdist2<-matrix(0,n,n)
- amdist<-array(NA,dim=c(n,m,lenl))
- mdist<-list()
- nam1<-names(lfdata)
- nam2<-names(lfdataref) 
- if (is.null(nam1)) {names(lfdata)<-nam1<-paste("var",1:lenl,sep="")}
- if (is.null(nam2)) {names(lfdataref)<-nam2<-paste("var",1:lenl2,sep="")} 
- if (missing(metric)){
-   if (is.fdata(lfdata[[nam1[1]]])) metric<-rep("metric.lp",len=lenl)
-   else    metric<-rep("metric.dist",len=lenl)   
- }  
-if (is.array(metric)) {
-     mdist=metric   
-     atr<-attributes(mdist)          
-     metric<-atr   #call(attributes(mdist)$call)[[1]]     
-      }
-else{ 
- dscale0<-rep(1,len=lenl)
- for (i in 1:lenl){
- if (is.null(par.metric[[nam1[i]]])) par.metric[[nam1[i]]]<-list()
- if (is.null(par.metric[[nam1[i]]]$dscale)) par.metric[[nam1[i]]]$dscale<-max    
- if (is.fdata(lfdata[[nam1[i]]])) { 
-   par.metric[[nam1[i]]][["fdata1"]]<-lfdata[[nam1[i]]]
-   par.metric[[nam1[i]]][["fdata2"]]<-lfdataref[[nam1[i]]] 
-  mdist<-do.call(metric[i],par.metric[[nam1[i]]])
-   }
-  else {
-   par.metric[[nam1[i]]][["x"]]<-lfdata[[nam1[i]]]
-   par.metric[[nam1[i]]][["y"]]<-lfdataref[[nam1[i]]]   
-   mdist<-do.call(metric[i],par.metric[[nam1[i]]])
-   }
-#  if (is.function(par.metric[[nam1[i]]]$dscale)) dscale0[i]<-par.metric[[nam1[i]]]$dscale(mdist)
-#  if (is.numeric(par.metric[[nam1[i]]]$dscale))  dscale0[i]<-par.metric[[nam1[i]]]$dscale
-#  else stop("Incorrect dscale argument for distance matrix")#dscale0[i]<-par.metric$dscale
-#  mdist<-mdist#/dscale0[i]
-#print(max(mdist))
-#print(dscale0)  
-  amdist[,,i]<- mdist         #si se devuelve ocupa mucho
-  #par.metric[[nam1[i]]]$dscale<-dscale0
- }
-
-# for (i in 1:lenl) { par.metric[[nam1[i]]]$dscale <- dscale0[i]}
-# print( amdist[1:4,1:4,i]  )
- atr<-attributes(mdist)        
-# print(amdist[,1,])
- mdist<-apply(amdist,1:2,method)
-# print((mdist))
-# print("fin1")
-}
+  lenl <- length(lfdata)
+  lenl2 <- length(lfdataref)
+  m0 <- nrow(lfdata[[1]])
+  if (is.null(rownames(lfdata[[1]]$data))) 
+    rownames(lfdata[[1]]$data) <- 1:m0
+  nms <- rownames(lfdata[[1]]$data)
+  nas <- NULL
+  for (i in 1:lenl) {
+    nas <- c(nas, na.action(na.omit(lfdata[[i]])))
+  }
+  nas <- unique(nas)
+  nullans <- !is.null(nas)
+  nam1<-names(lfdata)
+  if (missing(metric)){
+    if (is.fdata(lfdata[[nam1[1]]])) metric<-rep("metric.lp",len=lenl)
+    else    metric<-rep("metric.dist",len=lenl)   
+  }
+  mdist2<-metric.ldata(lfdata=lfdata,lfdataref=lfdataref,metric=metric,par.metric=par.metric,method=method)
+  if (equal) mdist<-mdist2
+  else mdist<-metric.ldata(lfdata=lfdataref,lfdataref=lfdataref,metric=metric,par.metric=par.metric,method=method)
+  m<-m0
+  n<-nrow(lfdata[[1]])
 }
 else stop("Error in lfdata argument")
-#attributes(mdist)<-atr
-attr(mdist, "par.metric") <-attr(atr, "par.metric")                                  
-attr(mdist, "call") <-attr(atr, "call")                                  
-attr(mdist, "method") <-method                                    
-# m2<-ncol(data2)
-# n2<-nrow(data2)
+#attr(mdist, "par.metric") <-attr(atr, "par.metric")                                  
+#attr(mdist, "call") <-attr(atr, "call")                                  
+#attr(mdist, "method") <-method                                    
 if (is.null(n) && is.null(m)) stop("ERROR IN THE DATA DIMENSIONS")
 #if (is.null(m) && is.null(m2)) stop("ERROR IN THE DATA DIMENSIONS")
-#if (is.matrix(metric)) mdist=metric                    
-#else  mdist=metric(fdataobj,fdataori,...)
-#class(mdist)<-"matrix"
-#print(h)
+#h="h=0.15"
 if (is.null(h))   {
-  h<-0.15
-  hq2=quantile(mdist,probs=h)
-  }                                   
-else hq2<-h 
-class(mdist)<-c("matrix","fdist")
-ans<-Ker.norm(mdist/hq2)    ####
+  h<-0.1
+  hq2=quantile(mdist,probs=h,na.rm=TRUE)
+#print("es nulo h")  
+}
+else {
+  #cat("no es nulo h ",h,"\n")    
+  if (is.numeric(h))    hq2<-h  
+  else hq2=quantile(mdist,probs=as.numeric(substring(h,first=3)),na.rm=TRUE)
+}
+#print(hq2)
+class(mdist2)<-class(mdist)<-c("matrix","fdist")
+ans<-Ker.norm(mdist2/hq2)    #### en (nxm) matrix (new X ref)
 #print(ans)
 ans<-apply(ans,1,sum,na.rm=TRUE)                                    
 #print(ans)
@@ -106,7 +64,7 @@ if (scale)   {
  k=which.max(ans)
  lista=which(ans>=quantile(ans,probs=trim,na.rm=TRUE))
 if (nullans) {
-  ans1<-rep(NA,len=m0)
+ ans1<-rep(NA,len=m0)
  ans1[-nas] <-ans 
  ans<-ans1    
  }       
@@ -123,10 +81,11 @@ if (draw){
    "2"={c(1,2)},
    "3"={c(1,3)},
    "4"={c(2,2)})            
-            par(mfrow =mf)                    }
+    par(mfrow =mf)                    
+   }
  names1<-names2<-names<-lfdata[[1]][["names"]]
  names1$main<-"depth.modep median"
-  tr<-paste("mode.tr",trim*100,"\u0025",sep="")                     
+ tr<-paste("mode.tr",trim*100,"\u0025",sep="")                     
  for (idat in 1:lenl) {
    data<-lfdata[[idat]]$data
    tt<-lfdata[[idat]]$argvals
@@ -139,7 +98,7 @@ if (draw){
    rownames(med$data)<-"modep.med"
    rownames(mtrim$data)<-tr
    ind1<-!is.nan(ans)
-   ans[is.nan(ans)]=NA
+   ans[is.nan(ans)]=NA     
    cgray=1-(ans-min(ans,na.rm=TRUE))/(max(ans,na.rm=TRUE)-min(ans,na.rm=TRUE))
    plot(lfdata[[idat]][ind1, ], col =  gray(cgray[ind1]),lty=1, main = paste(nam1[idat]," modep Depth",sep=""))
    lines(mtrim,lwd=2,col="yellow")
@@ -147,11 +106,40 @@ if (draw){
    legend("topleft",legend=c(tr,"Median"),lwd=2,col=c("yellow","red"),box.col=0)
  }
 }
+# print("fin modep")
 return(invisible(list("lmed"=k,"ltrim"=lista,"dep"=ans,"metric"=metric,
 "par.metric"=par.metric,"mdist"=mdist,"hq"=hq2)))
 } 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 ##################################################################################
 depth.mode=function(fdataobj,fdataori=fdataobj,trim=0.25,metric=metric.lp,h=NULL,scale=FALSE,
 draw=FALSE,...){    
@@ -190,22 +178,31 @@ n2<-nrow(data2)
 if (is.null(n) && is.null(m)) stop("ERROR IN THE DATA DIMENSIONS")
 if (is.null(m) && is.null(m2)) stop("ERROR IN THE DATA DIMENSIONS")
 if (is.matrix(metric)) mdist=metric                    
-else  mdist=metric(fdataobj,fdataori,...)
+else  mdist=metric(fdataori,fdataori,...)
 class(mdist)<-"matrix"
+
+if (n==n2 & m==m2) {
+  equal<-all(fdataobj$data==fdataori$data)
+  if (equal) mdist2<-mdist
+  else mdist2<-metric(fdataobj,fdataori,...)}
+else  mdist2<-metric(fdataobj,fdataori,...)
 if (is.null(h))   {
   h<-0.15
   hq2=quantile(mdist,probs=h,na.rm=TRUE)
   }
-else hq2<-h  
-class(mdist)<-c("matrix","fdist")
-ans<-Ker.norm(mdist/hq2)    ####
+else {
+  if (is.numeric(h))    hq2<-h  
+  else hq2=quantile(mdist,probs=as.numeric(substring(h,first=3)),na.rm=TRUE)
+}
+class(mdist)<-class(mdist2)<-c("matrix","fdist")
+ans<-Ker.norm(mdist2/hq2)    ####
 ans<-apply(ans,1,sum,na.rm=TRUE)                                    
 if (scale)   {
-   mdist2<-metric(fdataori,fdataori,...)
-   ans2<-Ker.norm(mdist2/hq2)    ####
-   ans2<-apply(ans2,1,sum,na.rm=TRUE)                                       
-   mn<-min(ans2,na.rm=TRUE)
-   mx<-max(ans2,na.rm=TRUE)
+#   mdist2<-metric(fdataori,fdataori,...)
+#   ans2<-Ker.norm(mdist2/hq2)    ####
+#   ans2<-apply(ans2,1,sum,na.rm=TRUE)                                       
+   mn<-min(ans,na.rm=TRUE)
+   mx<-max(ans,na.rm=TRUE)
 #   scl<-mx-mn
 #   ans=as.vector(scale(ans,center=mn,scale=scl))
 #   ans2=as.vector(scale(ans2,center=mn,scale=scl))
@@ -254,12 +251,16 @@ return(invisible(out))
 }
 ################################################################################
 mdepth.LD=function(x,xx=x,metric=metric.dist,h=NULL,scale=FALSE,...){    
- if (is.vector(xx)) stop("One of x or xx must be a matrix")
+ if (is.vector(x)){
+   if (all(xx==x)) x<-xx<-matrix(x,ncol=1)#stop("One of x or xx must be a matrix")
+   else   {
+ 	m2=ncol(xx)
+	if (length(x)!=m2) stop("Length of x does not match with dimension of xx")
+	x=matrix(x,ncol=m2)
+   }
+ }    
 		m2=ncol(xx)
 		n2=nrow(xx)
-if (is.vector(x)){
-	if (length(x)!=m2) stop("Length of x does not match with dimension of xx")
-	x=matrix(x,ncol=m2)}
 		n<-nrow(x)
 		m<-ncol(x)
 if (is.null(rownames(x)))  rownames(x)<-1:nrow(x)
@@ -272,22 +273,31 @@ if (is.null(rownames(x)))  rownames(x)<-1:nrow(x)
 if (is.null(n) && is.null(m)) stop("ERROR IN THE DATA DIMENSIONS")
 if (is.null(m) && is.null(m2)) stop("ERROR IN THE DATA DIMENSIONS")
 if (is.matrix(metric)) {mdist=metric}
-else {  mdist=metric(x,xx,...)  }
+else {  mdist=metric(xx,xx,...)  }
 class(mdist)<-"matrix"
-#hq=quantile(mdist+diag(Inf,nrow(mdist)),probs=h)
+
+if (n==n2 & m==m2) {
+  equal<-all(x==xx)
+  if (equal) mdist2<-mdist
+  else mdist2<-metric(x,xx,...)}
+else  mdist2<-metric(x,xx,...)
 if (is.null(h))   {
   h<-0.15
-  hq2=quantile(mdist,probs=h)
-  }
-else hq2<-h
-class(mdist)<-c("matrix")
-ans<-Ker.norm(mdist/hq2)    ####
-ans<-apply(ans,1,sum,na.rm=TRUE)
-mx=scale
-if (scale) { #en predcicion hay que usar los valores de scala usados previamente
-mx=max(ans,na.rm=TRUE)
-ans=as.vector(ans/mx)
-}                                    
+  hq2=quantile(mdist,probs=h,na.rm=TRUE)
+}
+else {
+  if (is.numeric(h))    hq2<-h  
+  else hq2=quantile(mdist,probs=as.numeric(substring(h,first=3)),na.rm=TRUE)  
+}
+class(mdist)<-class(mdist2)<-c("matrix")
+ans<-Ker.norm(mdist2/hq2)    ####
+ans<-apply(ans,1,sum,na.rm=TRUE)                                    
+mx = scale
+if (scale)   {
+#  mn<-min(ans,na.rm=TRUE)
+  mx<-max(ans,na.rm=TRUE)
+  ans=as.vector(ans/mx)   
+}                                
 if  (nullans){
         ans1<-rep(NA,len=n)
         ans1[-nas] <-ans 
@@ -297,6 +307,9 @@ names(ans)<-nms
 out<-list("dep" = ans,"hq"=hq2,dscale=mx)                                                                  
 return(invisible(out))
 }
+##########################################################################
+##########################################################################
+#out.p[[icls]]=classif.DD(yy,lxx,depth=dep.int[idep],classif=cls[icls])                  
 
+#d<-depth.modep(lxx,lx.test)
 
-  
\ No newline at end of file
