@@ -327,13 +327,11 @@ else {
     mat[-1,-1]<-lambda*P
     mat[1,1]<-0   
     }
-#     S<-solve(t(scores)%*%W%*%scores+mat)  
-
-    S=solve(t(Z)%*%Z+mat)
+    S<-t(Z)%*%Z+mat
+    S<-Minverse(S)
     H<-Z%*%S%*%t(Z)
     e<-object.lm$residuals
-#    df = traza(H)
-    df<-pc$df[lenl]+1
+    df = max(traza(H),pc$df[lenl]+1)
     rdf<-n-df
     sr2 <- sum(e^2)/rdf
     Vp<-sr2*S 
@@ -390,22 +388,21 @@ else {
       pii <-traza(P2)
       lambda0<-(-2*pii+sqrt(4*pii^2-4*(nc-normx^2)*normp^2))/(2*normp^2)
       #  lambda1<-normx/normp
-      #print(lambda0)
-      #print(lambda1)
       lambda<-seq(0,sqrt(lambda0),len=10)
     }
   }
-    lenrn<-length(lambda)
-    ind =1:kmax
-    l = l2 = list()
-    ck = 1
-    tab = list("AIC", "AICc","SIC", "SICc","HQIC","rho","CV")
-    type.i = pmatch(criteria, tab)
-    MSC.min<-Inf
-    cv.AIC <- matrix(NA,nrow=lenrn,ncol=kmax)
-    if (is.na(type.i))     stop("Error: incorrect criteria")
-    else {
-    if (type.i < 7) {
+  tol<-sqrt(.Machine$double.eps)
+  lenrn<-length(lambda)
+  ind =1:kmax
+  l = l2 = list()
+  ck = 1
+  tab = list("AIC", "AICc","SIC", "SICc","HQIC","rho","CV")
+  type.i = pmatch(criteria, tab)
+  MSC.min<-Inf
+  cv.AIC <- matrix(NA,nrow=lenrn,ncol=kmax)
+  if (is.na(type.i))     stop("Error: incorrect criteria")
+  else {
+  if (type.i < 7) {
 #        cv.AIC <- rep(NA, kmax)
       for (r in 1:lenrn) {    
         pls<-fdata2pls(fdataobj,y,ncomp=kmax,lambda=lambda[r],P=P,...)
@@ -422,7 +419,7 @@ else {
               "SICc"=log(s2) + log(n) * ck/(n-ck-2),
               "HQIC"=log(s2) + 2*log(log(n)) * ck/n,
               "rho"={A<-out$residuals;B<-1-diag(out$H)/n; D1<-(A/B)^2;sum(D1)})
-     if ( MSC.min>cv.AIC[r,j]) {
+     if ( MSC.min>(cv.AIC[r,j]+tol)) {
        rn.opt<-r
        pc.opt<-j
        MSC.min= cv.AIC[r,j]
@@ -468,283 +465,286 @@ else {
 }
 #################################################################
 #################################################################
+
 #################################################################
 #################################################################
 fregre.pc.cv=function (fdataobj, y, kmax=8,lambda=0,P=c(1,0,0),criteria = "SIC",weights=rep(1,len=n),...) {
-   sequen=FALSE
-   if (length(kmax)>1) {
-   sequen=TRUE
-   l<-kmax
-   kmax<-max(kmax)
-   }
+  sequen=FALSE
+  if (length(kmax)>1) {
+    sequen=TRUE
+    l<-kmax
+    kmax<-max(kmax)
+  }
   if (class(fdataobj)=="fdata.comp") {
-  fdataobj<-fdataobj$fdataobj
-  if (min(lambda)!=0 | !is.null(P)) warning("The arguments lambda and P are not used")
+    fdataobj<-fdataobj$fdataobj
+    if (min(lambda)!=0 | !is.null(P)) warning("The arguments lambda and P are not used")
   }
   else {
-   if (!is.fdata(fdataobj)) fdataobj=fdata(fdataobj)
-   tt<-fdataobj[["argvals"]]
-   x<-fdataobj[["data"]]
-   X<-fdata.cen(x)[[1]]$data
-   np<-ncol(x)
-   if (min(lambda)>0) {
-    if (is.logical(lambda[1]))   {
-      if (is.vector(P)) {    P2<-P.penalty(tt,P=P)   }
-      normx<-sqrt(sum(abs((t(x)%*%x)^2)))
-      normp<-sqrt(sum(abs(P2^2)))
-      normip<-sqrt(sum(abs(diag(np)+P2)^2))
-      pii <-traza(P2)
-      lambda0<-(-2*pii+sqrt(4*pii^2-4*(np-normx^2)*normp^2))/(2*normp^2)
-      #  lambda1<-normx/normp
-      #print(lambda0)
-      #print(lambda1)
-      lambda<-seq(0,sqrt(lambda0),len=10)
-    }
-  }
-  }
-#  pc<-fdata2ppc(fdataobj,ncomp=kmax,lambda=lambda,P=P,...)
-if (is.null(names(y))) names(y)<-1:length(y)
-rtt<-fdataobj[["rangeval"]]
-n <- nrow(x);    #nc <- ncol(x)
-cv.opt1 = Inf;    pc.opt1 = NA
-c1 = matrix(1:kmax, nrow = 1)
-num.pc = nrow(c1)
-l = l2 = list()
-max.c = length(c1)
-c0 = 1:kmax
-use = rep(FALSE, kmax)
-tab = list("AIC", "AICc","SIC", "SICc","rho","CV")
-type.i = pmatch(criteria, tab)
-#pc2<-pc
-lenrn<-length(lambda)
-MSC3<-list()
-pc.opt2 <- matrix(NA,nrow=lenrn,ncol=kmax)
-rownames(pc.opt2)<-paste("lambda=",signif(lambda,4),sep="")
-colnames(pc.opt2)<-paste("PC(",1:kmax,")",sep="")
-MSC2<-pc.opt2
-MSC.min<-Inf
-min.rn<-lambda[1]
-if (is.na(type.i))     stop("Error: incorrect criteria")
-else {
-   if (type.i < 6) {
-   for (r in 1:lenrn) {
-       pc<-fdata2pc(fdataobj,ncomp=kmax,lambda=lambda[r],P=P)
-#       pc<-fdata2pc(fdataobj,ncomp=kmax,...)
-#       if (!is.matrix(P)) if (is.vector(P)) P<-P.penalty(tt,P)
-       cv.opt1 = Inf
-       pc.opt1 = NA
-       l = l2 = list()
-       c1 = matrix(1:kmax, nrow = 1)
-       num.pc = nrow(c1)
-       max.c = length(c1)
-       c0 = 1:kmax
-       use = rep(FALSE, kmax)
-       pc2<-pc
-       for (k in 1:kmax) {
-                cv.AIC <- rep(NA, max.c)
-                if (sequen) {
-                   max.c=1
-                  c1<-matrix(pc$l[1:k],ncol=1)
-                }
-                for (j in 1:max.c) {
-                  pc2$rotation <- pc$rotation#[c1[, j]]
-                  pc2$l <- pc$l[c1[, j]]
-                  out = fregre.pc(pc2, y,l=c1[, j],lambda=lambda[r],P=P,weights=weights,...)
-                  ck<-out$df
-                  s2 <- sum(out$residuals^2)/n
-                  cv.AIC[j]<-switch(criteria,
-              "AIC"=log(s2) + 2 * (ck)/n,
-              "AICc"=log(s2) + 2 * (ck)/(n - ck - 2),
-              "SIC"=log(s2) + log(n) * ck/n,
-              "SICc"=log(s2) + log(n) * ck/(n-ck-2),
-              "HQIC"=log(s2) + 2*log(log(n)) * ck/n,
-              "rho"={A<-out$residuals;B<-1-diag(out$H)/n; D1<-(A/B)^2;sum(D1)})
-                }
-#                min.AIC = min(cv.AIC)
-#                pc.opt1 <- c1[, which.min(cv.AIC)]
-#                l[[k]] = pc.opt1[k]
-#                l2[[k]] = min.AIC
-#                use[pc.opt1[k]] = TRUE
-#                l[[k + 1]] = c0[use == FALSE]
-#                c1 = t(expand.grid(l))
-#                ck = nrow(c1) + 1
-#                max.c = ncol(c1)
-             if (!sequen){
-                min.AIC = min(cv.AIC)
-                pc.opt1 <- c1[, which.min(cv.AIC)]
-                l[[k]] = pc.opt1[k]
-                l2[[k]] = min.AIC
-                use[pc.opt1[k]] = TRUE
-                l[[k + 1]] = c0[use == FALSE]
-                c1 = t(expand.grid(l))
-                ck = nrow(c1) + 1
-                max.c = ncol(c1)
-                }
-         else {
-                pc.opt1 <- 1:k
-                l[[k]] = k
-                l2[[k]] =drop(cv.AIC[1])
-                }
-            }
-     mn = which.min(l2)
-     MSC = as.numeric(l2)
-     if ( MSC.min>MSC[mn]) {
-       min.rn<-r
-       MSC.min = MSC[mn]
-       pc.opt3<-pc.opt1[1:mn]
-       }
-    pc.opt = pc.opt1[1:mn]
-    MSC2[r,]<-MSC
-    pc.opt2[r,]<-pc.opt1
-    }
-    }
-#### CV criteria
-    else {
-    pb=txtProgressBar(min=0,max=lenrn,width=50,style=3)
-    pcl<-list()
-    for (r in 1:lenrn) {
-     setTxtProgressBar(pb,r-0.5)
-     for (i in 1:n) {
-      pcl[[i]]<-fdata2pc(fdataobj[-i,],ncomp=kmax,lambda=lambda[r],P=P)
+    if (!is.fdata(fdataobj)) fdataobj=fdata(fdataobj)
+    tt<-fdataobj[["argvals"]]
+    x<-fdataobj[["data"]]
+    X<-fdata.cen(x)[[1]]$data
+    np<-ncol(x)
+    if (min(lambda,na.rm=TRUE)>0) {
+      if (is.logical(lambda[1]))   {
+        if (is.vector(P)) {    P2<-P.penalty(tt,P=P)   }
+        normx<-sqrt(sum(abs((t(x)%*%x)^2)))
+        normp<-sqrt(sum(abs(P2^2)))
+        normip<-sqrt(sum(abs(diag(np)+P2)^2))
+        pii <-traza(P2)
+        lambda0<-(-2*pii+sqrt(4*pii^2-4*(np-normx^2)*normp^2))/(2*normp^2)
+        #  lambda1<-normx/normp
+        #print(lambda0)
+        #print(lambda1)
+        lambda<-seq(0,sqrt(lambda0),len=10)
       }
-     cv.opt1 = Inf
-     pc.opt1 = NA
-     l = l2 = list()
-     c1 = matrix(1:kmax, nrow = 1)
-     num.pc = nrow(c1)
-     max.c = length(c1)
-     c0 = 1:kmax
-     use = rep(FALSE, kmax)
-     for (k in 1:kmax) {
-      if (sequen) {   max.c=1;   c1<-matrix(pc$l[1:k],ncol=1)   }
-      cv.AIC <- rep(NA, max.c)
-      cv.AIC2 <- matrix(NA,nrow=max.c,ncol=lenrn)
-
-      rownames(cv.AIC2)<-1:max.c
-      for (j in 1:max.c) {
-        residuals2<- rep(NA, n)
-        maxk<-max(c1[, j])
-          for (i in 1:n){
-            pc2<-pcl[[i]]
-            pc2$rotation<-pcl[[i]]$rotation#[c1[,j]]
-            pc2$l<-pcl[[i]]$l[c1[,j]]
-            out = fregre.pc(pc2,y[-i],l=c1[,j],weights=weights[-i],...) #####
-            ck<-out$df
-            residuals2[i] <- ((y[i] - predict(out,fdataobj[i,]))/(n-ck))^2
-            }
-          cv.AIC[j] <-sum(residuals2)/n
+    }
+  }
+  #  pc<-fdata2ppc(fdataobj,ncomp=kmax,lambda=lambda,P=P,...)
+  if (is.null(names(y))) names(y)<-1:length(y)
+  rtt<-fdataobj[["rangeval"]]
+  n <- nrow(x);    #nc <- ncol(x)
+  cv.opt1 = Inf;    pc.opt1 = NA
+  c1 = matrix(1:kmax, nrow = 1)
+  num.pc = nrow(c1)
+  l = l2 = list()
+  max.c = length(c1)
+  c0 = 1:kmax
+  use = rep(FALSE, kmax)
+  tab = list("AIC", "AICc","SIC", "SICc","rho","CV")
+  type.i = pmatch(criteria, tab)
+  #pc2<-pc
+  lenrn<-length(lambda)
+  MSC3<-list()
+  pc.opt2 <- matrix(NA,nrow=lenrn,ncol=kmax)
+  rownames(pc.opt2)<-paste("lambda=",signif(lambda,4),sep="")
+  colnames(pc.opt2)<-paste("PC(",1:kmax,")",sep="")
+  MSC2<-pc.opt2
+  MSC.min<-Inf
+  min.rn<-lambda[1]
+  if (is.na(type.i))     stop("Error: incorrect criteria")
+  else {
+    if (type.i < 6) {
+      for (r in 1:lenrn) {
+        pc<-fdata2pc(fdataobj,ncomp=kmax,lambda=lambda[r],P=P)
+        #       pc<-fdata2pc(fdataobj,ncomp=kmax,...)
+        #       if (!is.matrix(P)) if (is.vector(P)) P<-P.penalty(tt,P)
+        cv.opt1 = Inf
+        pc.opt1 = NA
+        l = l2 = list()
+        c1 = matrix(1:kmax, nrow = 1)
+        num.pc = nrow(c1)
+        max.c = length(c1)
+        c0 = 1:kmax
+        use = rep(FALSE, kmax)
+        pc2<-pc
+        for (k in 1:kmax) {
+          cv.AIC <- rep(NA, max.c)
+          if (sequen) {
+            max.c=1
+            c1<-matrix(pc$l[1:k],ncol=1)
           }
-if (!sequen){
-                min.AIC = min(cv.AIC)
-                pc.opt1 <- c1[, which.min(cv.AIC)]
-                l[[k]] = pc.opt1[k]
-                l2[[k]] = min.AIC
-                use[pc.opt1[k]] = TRUE
-                l[[k + 1]] = c0[use == FALSE]
-                c1 = t(expand.grid(l))
-                ck = nrow(c1) + 1
-                max.c = ncol(c1)
-                }
-     else { pc.opt1 <- 1:k; l[[k]] = k;l2[[k]] =drop(cv.AIC[1]) }
+          for (j in 1:max.c) {
+            pc2$rotation <- pc$rotation#[c1[, j]]
+            pc2$l <- pc$l[c1[, j]]
+            out = fregre.pc(pc2, y,l=c1[, j],lambda=lambda[r],P=P,weights=weights,...)
+            ck<-out$df
+            s2 <- sum(out$residuals^2)/n
+            cv.AIC[j]<-switch(criteria,
+                              "AIC"=log(s2) + 2 * (ck)/n,
+                              "AICc"=log(s2) + 2 * (ck)/(n - ck - 2),
+                              "SIC"=log(s2) + log(n) * ck/n,
+                              "SICc"=log(s2) + log(n) * ck/(n-ck-2),
+                              "HQIC"=log(s2) + 2*log(log(n)) * ck/n,
+                              "rho"={A<-out$residuals;B<-1-diag(out$H)/n; D1<-(A/B)^2;sum(D1)})
+          }
+          #                min.AIC = min(cv.AIC)
+          #                pc.opt1 <- c1[, which.min(cv.AIC)]
+          #                l[[k]] = pc.opt1[k]
+          #                l2[[k]] = min.AIC
+          #                use[pc.opt1[k]] = TRUE
+          #                l[[k + 1]] = c0[use == FALSE]
+          #                c1 = t(expand.grid(l))
+          #                ck = nrow(c1) + 1
+          #                max.c = ncol(c1)
+          if (!sequen){
+            min.AIC = min(cv.AIC)
+            pc.opt1 <- c1[, which.min(cv.AIC)[1]]
+            l[[k]] = pc.opt1[k]
+            l2[[k]] = min.AIC
+            use[pc.opt1[k]] = TRUE
+            l[[k + 1]] = c0[use == FALSE]
+            c1 = t(expand.grid(l))
+            ck = nrow(c1) + 1
+            max.c = ncol(c1)
+          }
+          else {
+            pc.opt1 <- 1:k
+            l[[k]] = k
+            l2[[k]] =drop(cv.AIC[1])
+          }
+        }
+        mn = which.min(l2)[1]
+        MSC = as.numeric(l2)
+        if ( MSC.min>MSC[mn]) {
+          min.rn<-r
+          MSC.min = MSC[mn]
+          pc.opt3<-pc.opt1[1:mn]
+        }
+        pc.opt = pc.opt1[1:mn]
+        MSC2[r,]<-MSC
+        pc.opt2[r,]<-pc.opt1
+      }
+    }
+    #### CV criteria
+    else {
+      pb=txtProgressBar(min=0,max=lenrn,width=50,style=3)
+      pcl<-list()
+      for (r in 1:lenrn) {
+        setTxtProgressBar(pb,r-0.5)
+        for (i in 1:n) {
+          pcl[[i]]<-fdata2pc(fdataobj[-i,],ncomp=kmax,lambda=lambda[r],P=P)
+        }
+        cv.opt1 = Inf
+        pc.opt1 = NA
+        l = l2 = list()
+        c1 = matrix(1:kmax, nrow = 1)
+        num.pc = nrow(c1)
+        max.c = length(c1)
+        c0 = 1:kmax
+        use = rep(FALSE, kmax)
+        for (k in 1:kmax) {
+          if (sequen) {   max.c=1;   c1<-matrix(pc$l[1:k],ncol=1)   }
+          cv.AIC <- rep(NA, max.c)
+          cv.AIC2 <- matrix(NA,nrow=max.c,ncol=lenrn)
+          
+          rownames(cv.AIC2)<-1:max.c
+          for (j in 1:max.c) {
+            residuals2<- rep(NA, n)
+            maxk<-max(c1[, j])
+            for (i in 1:n){
+              pc2<-pcl[[i]]
+              pc2$rotation<-pcl[[i]]$rotation#[c1[,j]]
+              pc2$l<-pcl[[i]]$l[c1[,j]]
+              out = fregre.pc(pc2,y[-i],l=c1[,j],weights=weights[-i],...) #####
+              ck<-out$df
+              residuals2[i] <- ((y[i] - predict(out,fdataobj[i,]))/(n-ck))^2
             }
-     mn = which.min(l2)
-     MSC = as.numeric(l2)
-     if ( MSC.min>MSC[mn]) {
-       min.rn<-r
-       MSC.min = MSC[mn]
-       pc.opt3<-pc.opt1[1:mn]
-       }
-    pc.order<-names(MSC)
-    pc.opt = pc.opt1[1:mn]
-    MSC2[r,]<-MSC
-    pc.opt2[r,]<-pc.opt1
-    setTxtProgressBar(pb,r)
+            cv.AIC[j] <-sum(residuals2)/n
+          }
+          if (!sequen){
+            min.AIC = min(cv.AIC)
+            pc.opt1 <- c1[, which.min(cv.AIC)[1]]
+            l[[k]] = pc.opt1[k]
+            l2[[k]] = min.AIC
+            use[pc.opt1[k]] = TRUE
+            l[[k + 1]] = c0[use == FALSE]
+            c1 = t(expand.grid(l))
+            ck = nrow(c1) + 1
+            max.c = ncol(c1)
+          }
+          else { pc.opt1 <- 1:k; l[[k]] = k;l2[[k]] =drop(cv.AIC[1]) }
+        }
+        mn = which.min(l2)[1]
+        MSC = as.numeric(l2)
+        if ( MSC.min>MSC[mn]) {
+          min.rn<-r
+          MSC.min = MSC[mn]
+          pc.opt3<-pc.opt1[1:mn]
+        }
+        pc.order<-names(MSC)
+        pc.opt = pc.opt1[1:mn]
+        MSC2[r,]<-MSC
+        pc.opt2[r,]<-pc.opt1
+        setTxtProgressBar(pb,r)
+      }
+      close(pb)
     }
-    close(pb)
-    }
- mn = which.min(l2)
- MSC = as.numeric(l2)
- names(pc.opt3)<-paste("PC", pc.opt3, sep = "")
- rn.opt<-lambda[min.rn]
- fregre=fregre.pc(fdataobj,y,l=drop(pc.opt3),lambda=rn.opt,P=P,weights=weights,...)
- return(list("fregre.pc"=fregre,pc.opt = pc.opt3,lambda.opt=rn.opt,
- PC.order=pc.opt2,MSC.order=MSC2))
+    if (all(is.na(MSC))) stop("System is computationally singular: try with other number of basis elements")
+    mn = which.min(l2)[1]
+    MSC = as.numeric(l2)
+    names(pc.opt3)<-paste("PC", pc.opt3, sep = "")
+    rn.opt<-lambda[min.rn]
+    fregre=fregre.pc(fdataobj,y,l=drop(pc.opt3),lambda=rn.opt,P=P,weights=weights,...)
+    return(list("fregre.pc"=fregre,pc.opt = pc.opt3,lambda.opt=rn.opt,
+                PC.order=pc.opt2,MSC.order=MSC2))
+  }
 }
-}
-
 ####################################################################
 ####################################################################
 
-#############################################################
 #################################################################
 fregre.pc=function (fdataobj, y, l =NULL,lambda=0,P=c(1,0,0),weights=rep(1,len=n),...){
-if (class(fdataobj)=="fdata.comp") {
+  if (class(fdataobj)=="fdata.comp") {
     pc<-fdataobj
     fdataobj<-pc$fdataobj
     if (is.null(l))    {
-       l<-pc$l
-       }
+      l<-pc$l
+    }
     else if (length(l)>nrow(pc$rotation)) stop("Incorrect value for  argument l")
     x <- fdataobj[["data"]]
     tt <- fdataobj[["argvals"]]                                                    
-   }
-else {
- if (is.null(l)) l<- 1:3
- if (!is.fdata(fdataobj))    fdataobj = fdata(fdataobj)
-#  omit<-omit.fdata(fdataobj,y)
-#  fdataobj<-omit[[1]]
-#  y<-omit[[2]]
-  tt<-fdataobj[["argvals"]]
-  x<-fdataobj[["data"]]
-  pc<-fdata2pc(fdataobj,ncomp=max(l),lambda=lambda,P=P)
-}  
-    rtt <- fdataobj[["rangeval"]]
-    names <- fdataobj[["names"]]
-    n = nrow(x); np <- ncol(x);lenl = length(l)
-    if (is.null(rownames(x)))        rownames(x) <- 1:n
-    X <-xcen<-pc$fdataobj.cen
-    if (n != (length(y)))   stop("ERROR IN THE DATA DIMENSIONS")
-    C <- match.call()
-    ycen = y - mean(y)
-    vs<-t(pc$rotation$data[l,,drop=F])
-    scores<-Z<-(pc$x[,l,drop=F])
-    cnames<-colnames(pc$x)[l]
-    df<-lenl+1
-    J<-min(np,lenl)
-    ymean<-mean(y)
-    ycen<- y - ymean
-    W<-diag(weights) 
-if (is.logical(lambda)) {
-#   val<-log(.25*(pc$d[1]^2),base=2)
-   lambda<-.25*(pc$d[1]^2)#lambda<-c(0,2^seq(0,val,len=10))
-   }
-          
- if (lambda>0) {
+  }
+  else {
+    if (is.null(l)) l<- 1:3
+    if (!is.fdata(fdataobj))    fdataobj = fdata(fdataobj)
+    #  omit<-omit.fdata(fdataobj,y)
+    #  fdataobj<-omit[[1]]
+    #  y<-omit[[2]]
+    tt<-fdataobj[["argvals"]]
+    x<-fdataobj[["data"]]
+    pc<-fdata2pc(fdataobj,ncomp=max(l),lambda=lambda,P=P)
+  }  
+  rtt <- fdataobj[["rangeval"]]
+  names <- fdataobj[["names"]]
+  n = nrow(x); np <- ncol(x);lenl = length(l)
+  if (is.null(rownames(x)))        rownames(x) <- 1:n
+  X <-xcen<-pc$fdataobj.cen
+  if (n != (length(y)))   stop("ERROR IN THE DATA DIMENSIONS")
+  C <- match.call()
+  ycen = y - mean(y)
+  vs<-t(pc$rotation$data[l,,drop=F])
+  scores<-Z<-(pc$x[,l,drop=F])
+  cnames<-colnames(pc$x)[l]
+  df<-lenl+1
+  J<-min(np,lenl)
+  ymean<-mean(y)
+  ycen<- y - ymean
+  W<-diag(weights) 
+  if (is.logical(lambda)) {
+    #   val<-log(.25*(pc$d[1]^2),base=2)
+    lambda<-.25*(pc$d[1]^2)#lambda<-c(0,2^seq(0,val,len=10))
+  }
+  
+  if (lambda>0) {
     xmean<-pc$mean
     d<-pc$newd[l]
     D<-diag(d)
     diagJ<-diag(J)
-#    lenrn<-length(rn)
+    #    lenrn<-length(rn)
     scores<-cbind(rep(1,n),pc$x[,l])
     order.deriv<-0     
     if (!is.matrix(P)){
       if (is.vector(P)) {
-         for (i in 1:length(P))   {      if (P[i]!=0)         order.deriv<-i}
-          P<-P.penalty(tt,P)
-         
-         P<-t(vs)%*%P%*%vs 
-         P<-P*(diff(rtt)/(np -1))^(order.deriv*2-1)
-         }         }
+        for (i in 1:length(P))   {      if (P[i]!=0)         order.deriv<-i}
+        P<-P.penalty(tt,P)
+        
+        P<-t(vs)%*%P%*%vs 
+        P<-P*(diff(rtt)/(np -1))^(order.deriv*2-1)
+      }         }
     mat<-diag(J+1)
     mat[-1,-1]<-lambda*P
     mat[1,1]<-0
-     S<-solve(t(scores)%*%W%*%scores+mat)       
-     Cinv<-S%*%t(scores)%*%W         
-     coefs<-Cinv%*%y
-     yp<-drop(scores%*%coefs)
-     H<-scores%*%Cinv
-     df<-traza(H)
-     coefs<-drop(coefs)
+    Sb<-t(scores)%*%W%*%scores+mat
+    # S<-solve(Sb)       
+    #    S=solve(t(Z)%*%W%*%Z)    
+    s<-Minverse(Sb) 
+    Cinv<-S%*%t(scores)%*%W         
+    coefs<-Cinv%*%y
+    yp<-drop(scores%*%coefs)
+    H<-scores%*%Cinv
+    df<-traza(H)
+    coefs<-drop(coefs)
     names(coefs)<-c("Intercept",cnames)
     beta.est<-coefs[-1]*pc$rotation[l]
     beta.est$data<-colSums(beta.est$data)
@@ -755,33 +755,33 @@ if (is.logical(lambda)) {
     sr2 <- sum(e^2)/ rdf
     r2 <- 1 - sum(e^2)/sum(ycen^2)
     r2.adj<- 1 - (1 - r2) * ((n -    1)/ rdf)
-#    GCV <- sum(e^2)/(n - df)^2
-        object.lm = list()
-        object.lm$coefficients <- coefs
-        object.lm$residuals <- drop(e)
-        object.lm$fitted.values <- yp
-        object.lm$x<-scores 
-        object.lm$y <- y
-        object.lm$rank <- df
-        object.lm$df.residual <-  rdf
-        Z=cbind(rep(1,len=n),Z)
-        colnames(Z)[1] = "(Intercept)"
-        std.error = sqrt(diag(S) *sr2)
-        Vp<-sr2*S 
-        t.value = coefs/std.error
-        p.value = 2 * pt(abs(t.value), n - df, lower.tail = FALSE)
-        coefficients <- cbind(coefs, std.error, t.value, p.value)
-        colnames(coefficients) <- c("Estimate", "Std. Error",
-            "t value", "Pr(>|t|)")
-        class(object.lm) <- "lm"
+    #    GCV <- sum(e^2)/(n - df)^2
+    object.lm = list()
+    object.lm$coefficients <- coefs
+    object.lm$residuals <- drop(e)
+    object.lm$fitted.values <- yp
+    object.lm$x<-scores 
+    object.lm$y <- y
+    object.lm$rank <- df
+    object.lm$df.residual <-  rdf
+    Z=cbind(rep(1,len=n),Z)
+    colnames(Z)[1] = "(Intercept)"
+    std.error = sqrt(diag(S) *sr2)
+    Vp<-sr2*S 
+    t.value = coefs/std.error
+    p.value = 2 * pt(abs(t.value), n - df, lower.tail = FALSE)
+    coefficients <- cbind(coefs, std.error, t.value, p.value)
+    colnames(coefficients) <- c("Estimate", "Std. Error",
+                                "t value", "Pr(>|t|)")
+    class(object.lm) <- "lm"
     out <- list(call = C, beta.est = beta.est,coefficients=coefs,
-    fitted.values =yp,residuals = e,H=H,df = df,r2=r2,#GCV=GCV,
-    sr2 = sr2,Vp=Vp,l = l,lambda=lambda,fdata.comp=pc,lm=object.lm,
-    coefs=coefficients,fdataobj = fdataobj,y = y)
-##################################
-}
-else {
-#print("no rn")
+                fitted.values =yp,residuals = e,H=H,df = df,r2=r2,#GCV=GCV,
+                sr2 = sr2,Vp=Vp,l = l,lambda=lambda,fdata.comp=pc,lm=object.lm,
+                coefs=coefficients,fdataobj = fdataobj,y = y)
+    ##################################
+  }
+  else {
+    #print("no rn")
     response = "y"
     dataf<-data.frame(y,Z,weights)
     colnames(dataf)<-c("y",cnames,"weights")
@@ -793,20 +793,32 @@ else {
     beta.est$names$main<-"beta.est"
     beta.est$data <- matrix(as.numeric(beta.est$data),nrow=1)
     Z=cbind(rep(1,len=n),Z)
-    S=solve(t(Z)%*%W%*%Z)
+    #    S=solve(t(Z)%*%W%*%Z)    
+    S<-t(Z)%*%W%*%Z
+    s<-Minverse(S) 
     H<-Z%*%S%*%t(Z)
     e<-object.lm$residuals
-     df<-n- object.lm$df
-     sr2 <- sum(e^2)/(n - df)
-     Vp<-sr2*S 
-     r2 <- 1 - sum(e^2)/sum(ycen^2)
-#     r2.adj<- 1 - (1 - r2) * ((n -    1)/(n-df))
-#     GCV <- sum(e^2)/(n - df)^2
-  out <- list(call = C, coefficients=object.lm$coefficients,residuals = e,
-  fitted.values =object.lm$fitted.values,weights=weights,beta.est = beta.est,
-  df = df,r2=r2,sr2 = sr2,Vp=Vp,H=H, l = l,lambda=lambda,P=P,fdata.comp=pc,
-  lm=object.lm,XX=Z, fdataobj = fdataobj,y = y)
+    df<-traza(H)#n- object.lm$df
+    sr2 <- sum(e^2)/(n - df)
+    Vp<-sr2*S 
+    r2 <- 1 - sum(e^2)/sum(ycen^2)
+    #     r2.adj<- 1 - (1 - r2) * ((n -    1)/(n-df))
+    #     GCV <- sum(e^2)/(n - df)^2
+    out <- list(call = C, coefficients=object.lm$coefficients,residuals = e,
+                fitted.values =object.lm$fitted.values,weights=weights,beta.est = beta.est,
+                df = df,r2=r2,sr2 = sr2,Vp=Vp,H=H, l = l,lambda=lambda,P=P,fdata.comp=pc,
+                lm=object.lm,XX=Z, fdataobj = fdataobj,y = y)
   }
- class(out) = "fregre.fd"
- return(out)
+  class(out) = "fregre.fd"
+  return(out)
 }
+#############################################################
+
+
+
+
+
+
+
+
+
